/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIURI;

[uuid(6117cdf1-cb10-42a3-9901-4f1bab7ffa4d)]
interface nsIServiceWorkerManager : nsISupports
{
  // Returns a Promise
  nsISupports register(in nsIDOMWindow aWindow, in DOMString aScope, in DOMString aScriptURI);

  // Returns a Promise
  nsISupports unregister(in nsIDOMWindow aWindow, in DOMString aScope);

  // aTarget MUST be a ServiceWorkerContainer.
  [noscript] void AddContainerEventListener(in nsIURI aPageURI, in nsIDOMEventTarget aTarget);
  [noscript] void RemoveContainerEventListener(in nsIURI aPageURI, in nsIDOMEventTarget aTarget);

  // Testing
  DOMString getScopeForUrl(in DOMString path);
};

%{ C++
#define SERVICEWORKERMANAGER_CONTRACTID "@mozilla.org/serviceworkers/manager;1"
%}
