/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "domstubs.idl"

interface nsIDOMGeoPosition;

[scriptable, function, uuid(527E8B53-6F29-4B6A-8D04-5C1666A4C4C1)]
interface nsIDOMGeoPositionCallback : nsISupports {
  void handleEvent(in nsIDOMGeoPosition position);
};
