/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsINetworkZonePolicy.idl
 */

#ifndef __gen_nsINetworkZonePolicy_h__
#define __gen_nsINetworkZonePolicy_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIRequest; /* forward declaration */


/* starting interface:    nsINetworkZonePolicy */
#define NS_INETWORKZONEPOLICY_IID_STR "639a0797-7870-4a37-9110-57ea9f14783f"

#define NS_INETWORKZONEPOLICY_IID \
  {0x639a0797, 0x7870, 0x4a37, \
    { 0x91, 0x10, 0x57, 0xea, 0x9f, 0x14, 0x78, 0x3f }}

class NS_NO_VTABLE nsINetworkZonePolicy : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_INETWORKZONEPOLICY_IID)

  /* bool checkPrivateNetworkPermission (in nsIRequest aRequest); */
  NS_IMETHOD CheckPrivateNetworkPermission(nsIRequest *aRequest, bool *_retval) = 0;

  /* void setPrivateNetworkPermission (in nsIRequest aRequest, in bool aAllowed); */
  NS_IMETHOD SetPrivateNetworkPermission(nsIRequest *aRequest, bool aAllowed) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsINetworkZonePolicy, NS_INETWORKZONEPOLICY_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSINETWORKZONEPOLICY \
  NS_IMETHOD CheckPrivateNetworkPermission(nsIRequest *aRequest, bool *_retval); \
  NS_IMETHOD SetPrivateNetworkPermission(nsIRequest *aRequest, bool aAllowed); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSINETWORKZONEPOLICY(_to) \
  NS_IMETHOD CheckPrivateNetworkPermission(nsIRequest *aRequest, bool *_retval) { return _to CheckPrivateNetworkPermission(aRequest, _retval); } \
  NS_IMETHOD SetPrivateNetworkPermission(nsIRequest *aRequest, bool aAllowed) { return _to SetPrivateNetworkPermission(aRequest, aAllowed); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSINETWORKZONEPOLICY(_to) \
  NS_IMETHOD CheckPrivateNetworkPermission(nsIRequest *aRequest, bool *_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->CheckPrivateNetworkPermission(aRequest, _retval); } \
  NS_IMETHOD SetPrivateNetworkPermission(nsIRequest *aRequest, bool aAllowed) { return !_to ? NS_ERROR_NULL_POINTER : _to->SetPrivateNetworkPermission(aRequest, aAllowed); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsNetworkZonePolicy : public nsINetworkZonePolicy
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSINETWORKZONEPOLICY

  nsNetworkZonePolicy();

private:
  ~nsNetworkZonePolicy();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsNetworkZonePolicy, nsINetworkZonePolicy)

nsNetworkZonePolicy::nsNetworkZonePolicy()
{
  /* member initializers and constructor code */
}

nsNetworkZonePolicy::~nsNetworkZonePolicy()
{
  /* destructor code */
}

/* bool checkPrivateNetworkPermission (in nsIRequest aRequest); */
NS_IMETHODIMP nsNetworkZonePolicy::CheckPrivateNetworkPermission(nsIRequest *aRequest, bool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void setPrivateNetworkPermission (in nsIRequest aRequest, in bool aAllowed); */
NS_IMETHODIMP nsNetworkZonePolicy::SetPrivateNetworkPermission(nsIRequest *aRequest, bool aAllowed)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsINetworkZonePolicy_h__ */
